@@ -1,3 +1,4 @@
+pub use deadpool_lapin;
 pub use lapin;
 pub use lapin::message::{BasicReturnMessage, Delivery, DeliveryResult};
 pub use lapin::options::{
@@ -5,7 +6,7 @@ pub use lapin::options::{
     QueueDeclareOptions,
 };
 pub use lapin::types::{AMQPValue, FieldTable, LongString, ShortString};
-pub use lapin::{BasicProperties, Channel, Connection, ConnectionProperties, Consumer, Error, ExchangeKind, Queue};
+pub use lapin::{BasicProperties, Channel, ConnectionProperties, Consumer, Error, ExchangeKind, Queue};
 
 pub use crate::ops::*;
 pub use crate::session::AmqpSession;
